/* This program is Copyright 1995 by David Manthey.
/*
/* Code structure:
/*  Read in file/command line options
/*  Compute timing factor for the current computer
/*  Convert from WAV file data to PC Timer frequency data
/*  Print information about file
/*  Play sound
/*
/* Code use:
/*  You may use any or all of this code, provided you give credit for it in
/*  your code (for instance, in your comments), and email me (see address,
/*  below) telling me that you are using it.
/*
/* Memory models / Compilers:
/*  This code should work with most C compilers.  If you use something over
/*  than a large or huge memory model (small, DOS extended, or whatever), you
/*  will need to chance the definition of the tick variable. A simple way of
/*  eliminating this program is to replace the 'tick[0]' with 'clock()',
/*  #include <time.h>, and eliminate the tick variable.  This works, but
/*  tends to be slower, which adds some high frequency noise to the entire
/*  process.  You will also need to change the halloc function call to a
/*  malloc function call if using something other than a huge memory model.
/*
/* Questions:
/*  If you have any questions of how this works or why I wrote it or anything
/*  else, write to me:
/*     email:  Garlon4@aol.com        U.S. mail: David Manthey
/*                                               1605 Hutton St., Apt. 5
/*                                               Troy, New York 12180
/*
/* This code was written in about an hour on Saturday, December 30, 1995.  It
/*  was originally compiled with the QuickC compiler, huge memory model.   */

#include <malloc.h>                        /* Needed for huge memory model */
#include <stdio.h>
#include <stdlib.h>

main(int argc, char *argv[])
{
  FILE *fptr;
  unsigned char *buf;                          /* Buffer to store WAV file */
  long *tick=((long far *)0x46C);   /* Ptr to tick counter in 0x40 segment */
  long len, maxlen;                               /* File length variables */
  long i, j, k;                                        /* Generic counters */
  long t1=10, t2=30, a=0, b=0, c, wait;           /* Timing test variables */
  long bit16=0;
  float volume=5, hertz;

  if (argc<2 || argv[argc-1][1]=='?') {
	printf("WAV to PC speaker player.   Copyright 1995 by David Manthey.  Use freely.\n\n"
		   "Syntax:  SOUND wavefilename [volume [frequency [bits]]]\n\n"
		   "The file name must include the extension.  If the frequency or numbers of bits\n"
		   "(8 or 16) is incorrect, they can be manually specified.  Frequency is given in\n"
		   "Hertz.  Try volumes in the range of 1-10.\n");
	exit(0); }
  fptr = fopen(argv[1], "rb");
  if (!fptr) {
	printf("Can't open WAV file.\n");
	exit(0); }
  fseek(fptr, 0, SEEK_END);
  maxlen = ftell(fptr);                             /* Get the file length */
  rewind(fptr);
  buf = (char *)halloc(maxlen, 1);          /* Used with huge memory model */
/* Disable the previous line when using something other than a huge memory
/*  model, and enable the line below.                                      */
/*  buf = (char *)malloc((size_t)maxlen);                                  */

  if (!buf) {
	printf("Can't allocate enough memory for file.");
	exit(0); }
  len = 0;
  while (len<maxlen) {                 /* Read in segments for large files */
	len += fread(buf+len, 1, 32000, fptr);
	if (len!=maxlen && (len%32000)) {
	  printf("Error reading file.");
	  exit(0); } }
  fclose(fptr);
  hertz = ((unsigned int *)(buf+0x18))[0];      /* Get frequency from file */
  bit16 = (buf[0x22]==16);                     /* Determine if 16 bit file */
  if (argc>=3)
	sscanf(argv[2], "%f", &volume);                     /* Get user volume */
  if (argc>=4)
	sscanf(argv[3], "%f", &hertz);                   /* Get user frequency */
  if (argc>=5) {
	sscanf(argv[4], "%ld", &bit16);             /* Get user number of bits */
	bit16 = (bit16==16); }
  if (volume<=0 || hertz<=0) {
	printf("Invalid parameters.\n");
	exit(0); }

  i = tick[0];      /* Compute timing constants for correct playback speed */
  c = hertz/18.2;       /* The tick counter is updated 18.2 times a second */
  while (i==(j=tick[0]));                            /* Wait for next tick */
  while (j==(i=tick[0])) {                           /* Count for one tick */
	a ++;                                   /* Mimic our key routine below */
	outp(0x42, buf[0]); for (k=0; k<t1; k++);
	k = tick[0]; k++;   /* Make sure that the outp come at equal intervals */
	outp(0x42, buf[0]); for (k=0; k<t1; k++); }
  while (i==(j=tick[0])) {                       /* Count for another tick */
	b ++;
	outp(0x42, buf[0]); for (k=0; k<t2; k++);
	k = tick[0]; k++;
	outp(0x42, buf[0]); for (k=0; k<t2; k++); }
  wait = (a*(b*(t1-t2)-c*t1)+b*c*t2)/(c*(b-a));

  if (!bit16) {
	for (i=44; i<len; i++) {          /* Convert to PC speaker frequencies */
	  j = (buf[i]-128.)*volume+23.5;
	  if (j<1)   j = 1;                          /* Can't play zero values */
	  if (j>46)  j = 46;                 /* Higher makes an annoying whine */
	  buf[i] = j; } }
  else {
	for (i=22; i<len/2; i++) {                      /* Convert 16-bit file */
	  j = ((int *)buf)[i]*volume/256.+23.5;
	  if (j<1)   j = 1;
	  if (j>46)  j = 46;
	  buf[i+22] = j; }                         /* Converts to 8 bit values */
	len = len/2+22; }
													/* Report what we have */
  printf("Length: %4.2f sec   Frequency: %1.0f Hz   Bits: %ld   Delay factor: %ld\n",
		 (float)len/hertz, hertz, bit16*8+8, wait);

  outp(0x43, 0x90);                           /* Turn on timer 2 generator */
  outp(0x61, inp(0x61)|1);                        /* Gate timer to speaker */
  outp(0x61, inp(0x61)|2);                              /* Turn on speaker */
  for (i=44; i<len; i++) {               /* Start at 44 to skip WAV header */
	k = tick[0];                /* Use just as much time as the test loops */
	outp(0x42, buf[i]);                                  /* Play frequency */
	for (j=0; j<wait; j++);
	k = tick[0]; k++;   /* Make sure that the outp come at equal intervals */
	outp(0x42, buf[i]);             /* Twice prevents high freq. overtones */
	for (j=0; j<wait; j++); }
  outp(0x61, inp(0x61)&0xFD);                          /* Turn off speaker */

  free(buf);                     /* It's polite to free what was allocated */
}
